***********************************
* Browser
* 11/1/94
* By Tony Fardella (tonyf@crl.com)
*
* Read in a text file that contains the list of files or other lists
* and display them. Users can select items from the list and display
* either the contents of a file or items from another list on the screen.
* Interface is modeled after the Internet Gopher program. Also supported
* is listing of DOS directorys enabling the viewing of text files and the
* downloading of any file.
*
* 12/9/94 - Changed quit command to Q and added a dialog to wait for
* TF	    user response. ESC no longer used in the program
*
*           Added flags to control downloading.
*           If ,ND is found in OPTDATA then downloading turned off globaly.
*           Any line in a list begining with a period is a flag line.
*           The flag line will will override global flags for specific files.
*           Current valid flags: ND - download not allowed
*                                YD - download allowed
*
* 12/18/94 - Changed the procedure GetDir to honor the global download
* TF         flag (DL).
*
*            Added file types DIZ and PRG to the type that could be
*            displayed on-screen from a directory listing.
*
* 2/11/95 - Added TOP and BOTTOM commands to be able to quickly get to the
* TF        begining or end of list or document. Getting to the end of a long
*           document can take a while. You have been warned!
*
* 2/15/95 - Added a new list item to allow calling TBBS commands from within
* TF        Browser.
*
* 3/19/95 - Added "V" command to view archive files.
* TF
*           Added timeout so that program would quit after about 2.5 minutes.
*
* 4/8/95  - Add a simple search function for a single text file.
* TF
*           Removed the timeout function since it seemed to be causing some
*           problems.
*

set escape off
set typeahead to 0
set exclusive off

on error do ShowError with listPath

*
* Initialize our public variables
*

if .not. uansi()
    do ChkANSI
endif

* Starting line # and Text postion
Public lOffset, txtPos

* Pointer to current record and total records
Public numRecs, curRec

* Current page line, total lines this page, maximum # or lines per page
Public curLine, pageLines, maxLines

* Current page number, total pages
Public pageNum, totalPages

Public ON, OFF, scrTitle, uQuit, DL, eText

lOffset     = 4
maxLines    = 18
curLine     = 1
txtPos	    = 10
ON	    = .T.
OFF	    = .F.
stext   = .T.

*
* Make temporary database filenames based on unique line number
*
listdb = "LST"+ULINE()+".DBF"   && Contains current list of items
prevdb = "PRV"+ULINE()+".DBF"   && Holds paths to previous lists
newstruc = "news"+ULINE()+".DBF"   && Holds paths to previous lists
pstruc = "pst"+ULINE()+".DBF"   && Holds paths to previous lists


*
* Create the template for the "previous lists" database
*
* This is where we store the links to the previous list as the users
* moves to other lists.
*
create &pstruc
use &pstruc
append blank
replace field_name with "LSTPATH", field_type with "C", field_len with 64
append blank
replace field_name with "LSTITEM", field_type with "N", field_len with 4
append blank
replace field_name with "LSTPAGE", field_type with "N", field_len with 4

if file(prevdb)
    erase &prevdb
endif

*
* Now create the "previous lists" database from the template
*
use
create &prevdb from &pstruc
erase &pstruc
prevRecNo = 0

*
* Create a template for the current list. We'll keep this one around since
* we will be using it many times. It will be deleted when the user quits
* the program.
*
create &newstruc
use &newstruc
append blank
replace field_name with "LTEXT", field_type with "C", field_len with 72
append blank
replace field_name with "LPATH", field_type with "C", field_len with 64
append blank
replace field_name with "LTYPE", field_type with "N", field_len with 2
append blank
replace field_name with "DLFLAG", field_type with "L", field_len with 1

*
* Read the OPTDATA and pull out the file path and optional flags
*
offset = at(chr(38)+chr(38), optdata())
if "/" $ substr(optdata(), offset)
    flag = at("/", substr(optdata(), offset))
    listPath = trim(substr(optdata(), offset + 3, (offset + flag) - (offset + 3) - 1))
    if "ND" $ trim(substr(optdata(), offset + flag))
	DL = .F.
    endif
else
    listPath = trim(substr(optdata(), offset + 3))
    DL = .T.
endif

do ReadList with listPath

do_display = .T.
do TopStatus
curLine = 1
pageNum = 0

*
* Browser Main Event Loop - Handle User input
*
do while .t.

    if do_display
        do DrawList
        do Highlight with curLine, ON
        do_display = .F.
    endif

    clear typeahead
    key = 255
    key = inkey(0)

    *
    * Handle the function keys otherwise they cause problems
    *	F2 - F10 = (-1) - (-9)
    *
    if key < 0
    key = 255
    endif

    sel = upper(chr(key))
    do case

    *
    * Display the help screen on "?"
    *
    case sel = "?"
	do ShowHelp
	do_display = .T.

    *
    * Quit on Q
    *
    case sel = "Q"          && .or. key = 0 <-- Take this out for now.
	uQuit = .F.
	do UserQuit
	if uQuit
	    exit
	endif
	do_display = .T.

    *
    * Previous line on cursor up
    *
    case sel = chr(05)
	if curLine > 1
	    do Highlight with curLine, OFF
	    curLine = curLine - 1
	    do Highlight with curLine, ON
	else
	    if pageNum <> 0
		pageNum = pageNum - 1
		curLine = maxLines
		do_display = .T.
	    endif
	endif

    *
    * Next line on cursor down
    *
    case sel = chr(24)
	if curLine < pageLines
	    do Highlight with curLine, OFF
	    curLine = curLine + 1
	    do Highlight with curLine, ON
	else
	   if recno() <> numRecs
		pageNum = pageNum + 1
		curLine = 1
		do_display = .T.
	    endif
	endif

    *
    * Previous page on page up
    *
    case sel = chr(18)
	if pageNum > 0
	    pageNum = pageNum - 1
	    curLine = maxLines
	    do_display = .T.
	endif

    *
    * Next page on page down
    *
    case sel = chr(3)
       if pageNum < totalPages - 1
	    pageNum = pageNum + 1
	    curLine = 1
	    do_display = .T.
	endif

    *
    * Display the selected file if Cursor Right or Return
    *
    case sel = chr(4) .or. sel = chr (13)

	do case

    * Text item
    case LTYPE = 0
	    do ShowText with LPATH, DLFLAG
	    if uQuit
            exit
	    endif

    * List item
    case LTYPE = 1
	    nxtList = LPATH
	    use
	    use &prevdb
	    prevRecNo = prevRecNo + 1
	    if prevRecNo > reccount()
            append blank
	    else
            go prevRecNo
	    endif
	    replace LSTPATH with listPath
	    replace LSTITEM with curLine
	    replace LSTPAGE with pageNum
	    listPath = nxtList
	    curLine = 1
	    pageNum = 0
	    do ReadList with listPath

    * Directory item
	case LTYPE = 2
	    nxtList = LPATH
	    use
	    use &prevdb
	    prevRecNo = prevRecNo + 1
	    if prevRecNo > reccount()
            append blank
	    else
            go prevRecNo
	    endif
	    replace LSTPATH with listPath
	    replace LSTITEM with curLine
	    replace LSTPAGE with pageNum
	    listPath = nxtList
	    curLine = 1
	    pageNum = 0
	    do GetDir with listPath

    * TBBS item
    case LTYPE = 8
        mdata = upper(LPATH)
        mtype = substr(mdata,(at("TYPE",mdata) + 4),(at("OPTDATA",mdata)))
        mtype = val(ltrim(rtrim(mtype)))
        mopt = ltrim(crtrim(right(mdata,(at("OPTDATA",mdata) + 7))))
        @0,0 clear
        dotbbs type mtype optdata mopt

    * Binary item
	case LTYPE = 9
	    do Box3D with 10,18,16,62,0
        set color to R/W
	    @11,32 say "Binary File Type"
        set color to B/W
        @13,22 say "Unable to view this type of file."
	    @14,22 say "Would you like to download it? (Y/N)"
	    set color to W/
        key = inkey(0)
	    if key < 0
            key = 255
	    endif
        if upper(chr(key)) = "Y"
            if  DLFLAG
                dotbbs type 1 optdata LPATH + " /D/NBC/NB"
            else
                do NotAvail
            endif
	    endif

	endcase

	* Re-open the workspace file
    use &listdb

	* and re-draw the list
	do TopStatus
	do_display = .T.

    *
    * Display previous list, if there is one, on cursor left
    *
    case sel = chr(19)
	if prevRecNo > 0
	    use &prevdb
	    go prevRecNo
	    listPath = LSTPATH
	    curLine = LSTITEM
	    pageNum = LSTPAGE
	    prevRecNo = prevRecNo - 1
	    do ReadList with listPath

	    * Re-open the workspace file
	    use &listdb

	    * and re-draw the list
	    do TopStatus
	    do_display = .T.
	endif

    *
    * Download file if "D"
    *
    case sel = "D"
	if DLFLAG
	    dotbbs type 1 optdata LPATH + " /D/NBC/NB"

	    * Re-open the workspace file
	    use &listdb
	else
	    do NotAvail
	endif
	* and re-draw the list
	do TopStatus
	do_display = .T.

    *
    * Display the file information on "I"
    *
    case sel = "I"
	do GetInfo with LPATH
	do_display = .T.

    *
    * Jump to item number if user enters a number 1 - 9
    *
    case sel > "0" .and. sel < ":"
	do Box3D with  12,24,15,58,0
    set color to B/W,W+
	inNum = chr(lastkey())
	@13,27 say "Go to item #:      of " + ltrim(str(numRecs))
	theKey = 0
	do while .T.
	    @13,41 say inNum + " "
        theKey = inkey(0)
	    do case
		case theKey = 8 .or. theKey = 19    && Backspace
		    inNum =left(inNum, len(inNum) - 1)
		case theKey > 47 .and. theKey < 58  && it's a number
		    if len(inNum) <> 3
			inNum = inNum + chr(theKey)
		    endif
        case theKey = 13
		    exit
         case theKey = 0
            inNum = ""
            exit
	    endcase
	enddo
	inNum = val(inNum)
	set color to w/
	if inNum <= numRecs .and. inNum > 0
        pageNum = int((inNum - 1) / maxLines)
	    curLine = inNum - (pageNum * maxLines)
	endif
       do_display = .T.

    *
    * Goto the end of the list on "B"
    *
    case sel = "B"
    pageNum = int((numRecs - 1) / maxLines)
    curLine = numRecs - (pageNum * maxLines)
    do_display = .T.

    *
    * Goto the start of the list on "T"
    *
    case sel = "T"
    pageNum = 0
    curLine = 1
    do_display = .T.

    *
    * View archive file if "V"
	*
    case sel = "V"
    if LTYPE = 9
        fPath = substr(rtrim(LPATH),1, rat("\", rtrim(LPATH)) - 1)
        optStr = fPath + ' /NL/I:"E ' + rtrim(LTEXT) + '"'
        dotbbs type 34 optdata optStr
        wait "--< End of Archive. Press any key to continue. >--"

        * Re-open the workspace file
        use &listdb
        * and re-draw the list
        do TopStatus
        do_display = .T.
    endif

    endcase
enddo


*
* We're done so let's clean up our mess and leave
*
close all
fclose

if file(listdb)
   erase &listdb
endif
if file(newstruc)
    erase &newstruc
endif
if file(prevdb)
    erase &prevdb
endif
quit

******************************************************************
* Program Procedures

****************************
Procedure DrawList
*
* Input: none
*
* Output: Display "maxLines" lines from a list on the screen
*
Private dRec, scLine

@ 5,0 clear
scLine = 0
dRec = (pageNum * maxLines) + 1
go dRec
do while (scLine < maxLines) .or. dRec = numRecs
    set color to GR+/
    scLine = scLine + 1
    itemNum = ltrim(str(dRec))
    colNum = 3 - len(itemNum)
    @ lOffset + scLine,colNum say itemNum + "."
    set color to BG/
    do case
    case LTYPE = 0
	@lOffset + scLine,4 say "<TXT>"
    case LTYPE = 1
	@lOffset + scLine,4 say "<LST>"
    case LTYPE = 2
	@lOffset + scLine,4 say "<DIR>"
    case LTYPE = 8
    @lOffset + scLine,4 say "<BBS>"
    case LTYPE = 9
	@lOffset + scLine,4 say "<BIN>"
    endcase
    set color to W+/
    @lOffset + scLine,txtPos say LTEXT
    dRec = dRec + 1
    if dRec > numRecs
	exit
    endif
    skip
enddo
pageLines = scLine
do PageStatus
return

***********************************
Procedure Highlight
Parameters nItem, state
*
* Set the highlight of the given item on or off
*
* Input: nItem = line number to work on
*	 state = ON  - highlight item
*	       = OFF - unhighlight item
*
go ((pageNum * maxLines) + nItem)

if state
    set color to N/W
    @ lOffset + nItem, txtPos say rtrim(LTEXT)
else
    set color to W+/
    @ lOffset + nItem, txtPos say rtrim(LTEXT)
endif
set color to W/
return

********************************
Procedure TopStatus
*
* Draws the status box at the top of the screen
*
clear
do Box3D with 0,1,4,79,0
set color to R/W
@ 2,60 say "<Q>uit  <?> Help"
do Center with 1, scrTitle
set color to W/
return

********************************
Procedure PageStatus
*
* Updates the page number in the status bar
*
set color to R/W
@ 2,4 say "Page " + ltrim(str(pageNum + 1)) + " of " + ltrim(str(ceiling(totalPages))) + " "
set color to W/
return

****************************
Procedure ShowHelp
*
* Show a help dialog in the list view
*
do Box3D with 6,18,22,62,0
set color to R/W
do Center with 7, "Browser 1.3"
do Center with 8, "By Tony Fardella"
set color to B/W
do Center with 10, "Help"
@11,22 say "Cursor Up/Down      - Prev/Next Line"
@12,22 say "Page Up/Down        - Prev/Next Page"
@13,22 say "Enter/Cursor Right  - Select Item"
@14,22 say "Cursor Left         - Prev List"
@15,22 say "1-9  - Jump       T  - Top"
@16,22 say "D    - Download   B  - Bottom"
@17,22 say "I    - Info       V  - View Archive"
@18,22 say "Q    - Quit"
set color to R/W
do Center with 20, "Press any key to continue"
wait ""
set color to w/
return

****************************
Procedure GetInfo
Parameters filePath
*
* Display file information including name, size, date and type
*
Private filename

do Box3D with 9,20,19,60,0
set color to R/W
@10,31 say "File Information"
set color to B/W

if LTYPE = 0 .or. LTYPE = 1 .or. LTYPE = 2 .or. LTYPE = 9
    filename = substr(filePath,rat("\",filePath) + 1,len(filePath) - rat("\",filePath))
    @12,24 say "Name: " + trim(filename)
    @13,24 say "Size: " + ltrim(str(fsize(filePath))) + " Bytes"
    @14,24 say "Date: " + dtoc(fdate(filePath))
else
    @12,24 say "Name: N/A"
    @13,24 say "Size: N/A"
    @14,24 say "Date: N/A"
endif

do case
case LTYPE = 0
    @15,24 say "Type: Text"
case LTYPE = 1
    @15,24 say "Type: List"
case LTYPE = 2
    @15,24 say "Type: Directory"
case LTYPE = 8
    @15,24 say "Type: TBBS"
case LTYPE = 9
    @15,24 say "Type: Binary"
endcase
set color to R/W
@17,27 say "Press any key to continue"
wait ""
set color to w/
return

******************************
Procedure ReadList
Parameters listPath
*
* Read the lines from a text file that contains a Browser formated list into
* a new list database.
*

if file(listdb)
    erase &listdb
endif

scrTitle = ""
use
create &listdb from &newstruc
use &listdb

*
* open the text file in read mode, and report an error if there was one
*
fopen handle (listPath) 10 2048
if handle = -1
    do ShowError with listPath
    close all
    fclose
    quit
endif

*
* Read the list into our new database file
*
count = 1
do while count <> 0
    * read a line from the file
    flread handle count displ
	if count <> 0

	    do case

	    case left(displ,1) = ";"
	    * This is a comment so we just ignore it

	    case left(displ,1) = "~"
	    * This is a is the title for our screen
		scrTitle = crtrim(right(displ,len(displ) - 1))

	    case left(displ,1) = "!"
	    * This is a text item
		append blank
		replace LTEXT with crtrim(right(displ,len(displ) - 1))
		replace LTYPE with 0
		replace DLFLAG with DL

	    case left(displ,1) = "$"
	    * This is a list item
		append blank
		replace LTEXT with crtrim(right(displ,len(displ) - 1))
		replace LTYPE with 1
		replace DLFLAG with .F.

	    case left(displ,1) = "%"
	    * This is a binary item
		append blank
		replace LTEXT with crtrim(right(displ,len(displ) - 1))
		replace LTYPE with 9
		replace DLFLAG with DL

	    case left(displ,1) = "^"
	    * This is a directory item
		append blank
		replace LTEXT with crtrim(right(displ,len(displ) - 1))
		replace LTYPE with 2
		replace DLFLAG with .F.

        case left(displ,1) = "&"
        * This is a TBBS menu item
		append blank
		replace LTEXT with crtrim(right(displ,len(displ) - 1))
        replace LTYPE with 8
		replace DLFLAG with .F.

        case left(displ,1) = "#"
	    * This is the filepath for the previous item
		replace LPATH with crtrim(right(displ, len(displ) -1))

	    case left(displ,1) = "."
	    * This line contains flags
		* If the line contains a "YD" then download is allowed
		* If the line contains a "ND" then download is not allowed
		do case
		case "YD" $ upper(displ)
		    replace DLFLAG with .T.

		case "ND" $ upper(displ)
		    replace DLFLAG with .F.

		endcase

	    endcase

	endif
enddo
fclose(handle)

goto top
numRecs = reccount()
totalPages = numRecs/maxLines

return

******************************
* Get Directory
*
Procedure GetDir
Parameters gdPath
*
* Create a list database from a DOS directory
*

private fName, dta

if file(listdb)
    erase &listdb
endif

scrTitle = crtrim(trim(gdPath))

use
create &listdb from  &newstruc
use &listdb

fName = findfirst(dta, gdPath, ".XXXX")
do while len(fName) <> 0
    fType = substr(fName,rat(".",fName) + 1,len(fName) - rat(".",fName))
    fType = upper(fType)

    do case
    case at(ftype,"TXT DOC BAT PRG DIZ") <> 0
	append blank
	replace LTEXT with fName
	replace LTYPE with 0
        replace DLFLAG with DL

    otherwise
	append blank
	replace LTEXT with fName
	replace LTYPE with 9
        replace DLFLAG with DL

    endcase

    fPath = left(gdPath,rat("\", gdPath)) + fName
    replace LPATH with fPath
    fName = findnext(dta)
enddo

goto top
numRecs = reccount()
totalPages = numRecs/maxLines

return

*******************************
Procedure UserQuit
*
* Bring up a dialog box asking if the user really wants to quit.
*
do Box3D with 8,18,13,62,0
set color to R/W
@9,37 say "Quit?"
set color to B/W
@11,22 say "Are you sure you want to quit? (Y/N)"
key = inkey(0)
if upper(chr(key)) = "Y" .or. key = 0
    uQuit = .T.
endif
set color to w/
return

*******************************
Procedure NotAvail
*
* Inform the user that the current item is not available for downloading.
*
do Box3D with 8,15,15,65,0
set color to R/W
@9,37 say "Sorry"
set color to B/W
@11,18 say "This item is not available for downloading."
@13,27 say "Press any key to continue"
wait ""
set color to w/
return

************************************
* ShowText
*
* Display a text file to the screen. Allows the user to page up and down
* through the file. Also allows the file to be downloaded.
*

***********************************
* Display a text file on the screen
*
Procedure ShowText
Parameters filePath, dl

*
* Make temporary database filenames based on unique line number
*
txtdb = "TR"+ULINE()+".DBF"
txtstruc = "TXT"+ULINE()+".DBF"

*
* if an old version of that file exists, delete it
*
if file(txtdb)
	erase &txtdb
endif

*
* Create the database in real time
*
create &txtstruc
use &txtstruc
append blank
replace field_name with "FPOS", field_type with "N", field_len with 10
use
create &txtdb from &txtstruc
erase &txtstruc

*
* Use the database we just created
*
use &txtdb

*
* Get the filename from the file path string
*
filename = substr(filePath,rat("\",filePath) + 1,len(filePath) - rat("\",filePath))

*
* Set initial value for file position
*
pos = 0


*
* Open the text file in read mode, and report an error if there was one
*
fopen textHandle (filePath) 10 2048
if textHandle = -1
	do ShowError with filePath
	return
endif


*
* Get the size of the file we're reading so we can display the
* percentage done
*
filesize = fsize(filePath)

*
* Establish the initial value for variables
*
txtLines = 19
counter = 1
file_end = .F.
do_display = .T.

srch = .F.
srchStr = ""

*
* do initial screen prep
*
do TextTopStat

*
* Get and store our initial file offset to our temp database
*
fseek textHandle pos 0
tpage = 1
counter = 1
append blank
replace FPOS with pos


*
* Showtext Main Event Loop - Handle user input
*
do while .t.

	if do_display
		do DrawScreen
		do_display = .F.
	endif

    key = 255
    key = inkey(0)

	*
	* Handle the function keys
	*	F2 - F10 = -1 - -9
	*
	if key < 0
        key = 255
	endif

	sel = upper(chr(key))
	do case

	*
	* Display the help screen on F1
	*
	case sel = "?"
		do TextHelp
		do_display = .T.

	*
	* Go back to the list on cursor left
	*
	case sel = chr(19)
		exit

	*
	* Quit to TBBS on Q
    case sel = "Q" .or. key = 0
		uQuit = .F.
		do UserQuit
		if uQuit
			exit
		endif
		do_display = .T.

	*
	* Previouse page on page up or cursor up
	*
	case sel = chr(05) .or. sel = chr(18)
		if tpage > 1
			tpage = tpage - 1
			do_display = .T.
		else
			do_display = .F.
		endif

    *
	* Next page on page down or cursor down
	*
	case sel = chr(24) .or. sel = chr(3)
		if .not. file_end
			tpage = tpage + 1
			if reccount() < tpage
				fseek textHandle scratch 0
				append blank
				replace FPOS with scratch
			endif
			do_display = .T.
		else
			do_display = .F.
		endif

	*
	* Download file if "D"
	*
	case sel = "D"
		if dl
			dotbbs type 1 optdata filePath+" /D/NBC/NB"

			* Re-open the workspace file
			use &txtdb
		else
			do NotAvail
		endif

		do TextTopStat
		do_display = .T.

	*
	* Goto last page if "B"
	*
	case sel = "B"
		if .not. file_end
			do EndDoc
			do_display = .T.
		endif

	*
	* Goto first page if "T"
	*
	case sel = "T"
		tpage = 1
		do_display = .T.

    *
    * Search if "S"
	*
    case sel = "S"
        do TextSrch
        do_display = .T.

    *
    * Search Again if "A"
	*
    case sel = "A"
        if srch .and. (.not. file_end)
			tpage = tpage + 1
			if reccount() < tpage
				fseek textHandle scratch 0
				append blank
				replace FPOS with scratch
            endif
            do FindText with srchStr
        endif

    endcase

enddo

*
* If we're here, we're done, so clean up and quit
*
use
fclose(textHandle)

if file(txtdb)
	erase &txtdb
endif

return

************************************************************************
* Showtext Procedures

********************************
Procedure TextTopStat
*
* Draws the status box at the top of the screen
* with the name of the file we're currently viewing.
*
clear
do Box3D with 0,1,4,79,0
set color to R/W
do Center with 1, trim(filename)
set color to W/
return

********************************
Procedure TextStat
Parameters end_flag 	&& if .T. then display 100% else display XX%
*
* Display the page # and percent done.
* If end_flag is .T. then at end of file so display 100%
*
*
* get our current file offset, then calculate the percentage done

*
fseek textHandle moof 0
percent = (moof * 100) / filesize

set color to R/W
@ 2,4 say "Page "+ ltrim(str(tpage)) + "  "
if end_flag
	@ 2,16 say "(100%)"
else
    @ 2,16 say "("+ltrim(str(percent))+"%)  "
endif
@ 2,60 say "<Q>uit  <?> Help"
set color to W/
return

****************************
Procedure DrawScreen
*
* Draws a page of text on the screen and updates the
* top status line.
*

@ 5,0 clear
file_end = .F.
counter = 1
hlton = chr(27) + "[46;30m"
hltoff = chr(27) + "[37;40m"

goto tpage
fseek textHandle new FPOS 0
do while counter < txtLines
	* read a line from the file
	flread textHandle count displ

	* if we actually read any characters, display them and loop
	if count <> 0

		* Filer out control characters
		cnum = count
		do while cnum <> 0
			c = substr(displ,cnum,1)
			if asc(c) < 32
				displ = stuff(displ,cnum,1," ")
			endif
			cnum = cnum - 1
		enddo

        if srch .and. upper(srchStr) $ upper(displ)
            srchPos = at(upper(srchStr),upper(displ))
            stxt = left(displ, srchPos - 1)
            stxt = stxt + hlton + substr(displ, srchPos, len(srchStr)) + hltoff
            stxt = stxt + right (displ, len(displ) - (srchPos + len(srchStr)) + 1)
            ? left(stxt, 95)
        else
            ? left(displ,79)
        endif

		counter = counter + 1
	else
		file_end = .T.
        counter = txtLines
	endif
enddo
do TextStat with file_end
return

****************************
Procedure EndDoc
*
* Move to the last page of the document
*

do PleaseWait
if .not. file_end
    tpage = reccount()
    goto tpage
endif

do while .not. file_end
	counter = 1
	fseek textHandle new FPOS 0
    do while counter < txtLines
		* read a line from the file
		flread textHandle count displ

		if count <> 0
			counter = counter + 1
		else
			file_end = .T.
            counter = txtLines
		endif
	enddo

    if .not. file_end
        tpage = tpage + 1
        if reccount() < tpage
            fseek textHandle scratch 0
            append blank
            replace FPOS with scratch
        endif
        goto tpage
    endif

enddo
return

****************************
Procedure TextHelp
*
* Display a help dialog to the user
*
do Box3D with 6,18,20,62,0
set color to R/W
do Center with 7, "Browser 1.3"
do Center with 8, "By Tony Fardella"
set color to B/W
@10,38 say "Help"
@11,22 say "Cursor Up/Down  - Previous/Next Page"
@12,22 say "Cursor Left     - Return to List"
@14,22 say "S - Search      A - Search Again"
@15,22 say "T - Top         B - Bottom"
@16,22 say "D - Download    Q - Quit"
set color to R/W
do Center with 18, "Press any key to continue"

wait ""
set color to w/
return

**********************************
* Text Search
*
Procedure TextSrch
*
*
*

do Box3D with  12,15,19,65,0
set color to R/W
do Center with 13, "Search"
set color to B/W
@15,19 say "Search for:"
set color to R/W
do Center with 16, "<ENTER> Accept            <ESC> Cancel"
eText = ""
do LineEdit with 15, 31, 60
if len(eText) > 0
    srchStr = eText
    srch = .T.
    tpage = 1
    do FindText with srchStr
else
    srchStr = ""
    srch = .F.
endif
set color to w/
return

********************************
* Find Text String
*
* Find a text string in the currently open text file
*
* Input: srchStr = text to search for
*
* Output: tpage = page text is found or eof if not found.
*
Procedure FindText
Parameters srchStr

do PleaseWait
goto tpage
if tpage < reccount()
    file_end = .F.
endif
do while .not. file_end
	counter = 1
	fseek textHandle new FPOS 0
    do while counter < txtLines
		* read a line from the file
		flread textHandle count displ

        * If when find the search string were're done
        if upper(srchStr) $ upper(displ)
            return
        endif

		if count <> 0
			counter = counter + 1
		else
			file_end = .T.
            counter = txtLines
		endif
	enddo

    if .not. file_end
        tpage = tpage + 1
        if reccount() < tpage
            fseek textHandle scratch 0
            append blank
            replace FPOS with scratch
        endif
        goto tpage
    endif
enddo
set color to w/
return

**********************************
* Misc. Procedures
*
* This file contains various procedures which can be used by any
* program.
*
* Included are: Center      - Center a text string on the screen
*               Box3D       - Draw a 3D looking ANSI box of any size
*               ChkANSI     - Check for ANSI terminal compatibility
*               ShowError   - Bring up a box showing a TDBS error message
*                             (requires Box3D)
*               LineEdit    - Input a line of text.
*

********************************
Procedure Center
Parameters nRow, cText
*
* Centers text string on the screen
*
* Input: nRow = screen row
*        cText = text string
*

Private nCol
nCol = Uwidth() - Len(cText)
nCol = Max(nCol,0)
nCol = nCol / 2
If Uansi()
    @nRow,nCol say cText
else
    ? Replicate(" ",nCol) + cText
endif
return

***************************
Procedure Box3D
Parameters orgRow, orgCol,  endRow, endCol, bStyle

Private nRow
*
* Create a "3D" box of the size passed by the input parameters
*
* Input: orgRow, orgCol = starting point
*        endRow, endCol = ending point
*        bStyle = Box Style 0 = recessed  1 = extruded

set intensity off
set color to N/W
@orgRow,orgCol say " "
if bStyle = 0
    set color to  N+/W
else
    set color to  W+/W
endif
@orgRow,orgCol+1 say "" + replicate("",(endCol - orgCol) - 4)
if bStyle = 0
    set color to W+/W
else
    set color to  N+/W
endif

@orgRow,endCol-2 say " "
set color to N+/N
@orgRow,endCol say ""

nRow = orgRow + 1
do while nRow < endRow - 1
    set color to N/W
    @nRow,orgCol say " "
    if bStyle = 0
        set color to  N+/W
    else
        set color to  W+/W
    endif
    @nRow,orgCol+1 clear to nRow,endCol
    @nRow,orgCol+1 say ""
    if bStyle = 0
        set color to  W+/W
    else
        set color to  N+/W
    endif
    @nRow,endCol-2 say ""
    set color to N+/N
    @nRow,endCol say ""
    nrow = nRow + 1
enddo

set color to N/W
@endRow-1,orgCol say " "
if bStyle = 0
    set color to  N+/W
else
    set color to  W+/W
endif
@endRow-1,orgCol+1 say ""
if bStyle = 0
    set color to  W+/W
else
    set color to  N+/W
endif
@endRow-1,orgCol+2 say replicate("",(endCol - orgCol) - 4) + " "
set color to N+/N
@endRow-1,endCol say ""

set color to N+/N
@endRow,orgCol say " "  + replicate("",endCol - orgCol)
return

***************************************
* Check for ANSI support
*
Procedure ChkANSI
*
* Check for ANSI compatibility
*

Private TESTANSI, TESTIBM, DUMMY, KEY, TEST

TESTANSI = UANSI()
TESTIBM = UIBMG()

IF .NOT. TESTANSI
    DUMMY = ULREPLACE(UANSI, .T.)
    ? CHR(27)+"[0m"
    DUMMY = ULREPLACE(UANSI, .F.)

    CLEAR

    ? ""
    ? "Testing for ANSI..."
    ? ""
    DUMMY = ULREPLACE(UANSI, .T.)
    ? CHR(27)+"[0m"+CHR(27)+"[31mC"+CHR(27)+"[33;1mo"+CHR(27)+"[32ml"+CHR(27)+"[34;1mo"+CHR(27)+"[35mr"+CHR(27)+"[0m"
    DUMMY = ULREPLACE(UANSI, .F.)
    ? ""
    ? "Does the above line look like:"
    ? ""
    ? "     1 - Unreadable junk"
    ? "     2 - Colorful text"
    ? "     3 - Monochrome text, some letters brighter than others"
    ? ""
    ? "Your selection... "
    TEST = "0"
    DO WHILE TEST = "0"
        KEY = INKEY(0)
        TEST = CHR(KEY)
        DO CASE
            CASE TEST = "1"
                ? ""
                ? "Sorry, this program requires ANSI terminal support."
                ? "Press and key to continue..."
                KEY = INKEY(5)
                quit
            CASE TEST = "2"
                ? ""
                ? "Your settings will be changed so you can see color online."
                ? "These changes will take effect on your next call to the system."
                DUMMY = ULREPLACE(UANSI, .T.)
            CASE TEST = "3"
                ? ""
                ? "Your settings will be changed so you can see bold text online."
                ? "These changes will take effect on your next call to the system."
                DUMMY = ULREPLACE(UANSI, .T.)
            OTHERWISE
                TEST = "0"
        ENDCASE
    ENDDO
ENDIF

IF .NOT. TESTIBM
    ? ""
    ? "Testing for IBM Graphics..."
    ? ""
    DUMMY = ULREPLACE(UIBMG, .T.)
    ? CHR(176)+CHR(177)+CHR(178)+CHR(219)
    DUMMY = ULREPLACE(UIBMG, .F.)
    ? ""
    ? "On the above line, do you see:"
    ? ""
    ? "     1 - The characters '012['"
    ? "     2 - Boxes with varying shades"
    ? ""
    ? "Your selection... "
    TEST = "0"
    DO WHILE TEST = "0"
        KEY = INKEY(0)
        TEST = CHR(KEY)
        DO CASE
            CASE TEST = "1"
                ? ""
                ? "Your settings will not be changed."
            CASE TEST = "2"
                ? ""
                ? "Your settings will be changed so you can see color online."
                ? "These changes will take effect on your next call to the system."
                DUMMY = ULREPLACE(UIBMG, .T.)
            OTHERWISE
                TEST = "0"
        ENDCASE
    ENDDO
ENDIF

IF TESTANSI .AND. TESTIBM
    ? ""
    ? "Testing not necessary.  ANSI and IBM Graphics already enabled."
    ? "Press any key..."
    KEY = INKEY(5)
ELSE
    ? ""
    ? "Changes (if any) complete.  Press any key..."
    KEY = INKEY(5)
ENDIF
return

*******************************
Procedure ShowError
Parameters thePath

do Box3D with 8,10,16,70,0
set color to R+/W
@9,37 say "Error !"
set color to B/W
@11,13 say "   Error: "+MESSAGE(FERROR())
@12,13 say "File was: [" + trim(thePath) + "]"
set color to R/W
@14,27 say "Press any key to continue"
wait ""
set color to w/
return

**********************************
Procedure PleaseWait
*
* Display a box on the screen requesting the user to Please Wait.
*
do Box3D with 6,25,9,55,0
set color to RB/W
@7,33 say "Please wait..."
set color to w/
do_display = .T.
return

*********************************
* Line Edit
*
* Input a line of text. Requires public variable eText
* be previously declared.
*
* Input: orgRow = Row to start input
*        orgCol = Column to start input
*        endCol = Column for end of input
*        eText = Whatever is in this variable will be put on the screen
*
* Output: eText = edited text
*
Procedure LineEdit
Parameters orgRow, orgCol, endCol

set color to W+/N
@orgRow,orgCol clear to orgRow,endCol
@orgRow,orgCol say eText

nRow = orgRow
nCol = orgCol + len(eText)
maxLen = endCol - orgCol

do while .T.
    key = inkey(0)
    if key < 0
        key = 255
    endif

    sel = chr(key)

    do case

    *
    * Right cursor
    *
    case sel = chr(4)
    if nCol < endCol .and. (nCol - orgCol) < len(eText)
        nCol = nCol + 1
        @nRow,nCol say ""
    endif

    *
    * Left Cursor
    *
    case sel = chr(19)
    if nCol > orgCol
        nCol = nCol - 1
        @nRow,nCol say ""
    endif

    *
    * Back space
    *
    case sel = chr(8)
        if nCol > orgCol
            nCol = nCol - 1
        endif
        eText = stuff(eText,(nCol - orgCol)+1,1,"")
        @orgRow,orgCol clear  to orgRow,endCol
        @orgRow,orgCol say eText
        @nRow,nCol say ""

    *
    * Delete
    *
    case sel = chr(127)
        eText = stuff(eText,(nCol - orgCol)+1,1,"")
        @orgRow,orgCol clear  to orgRow,endCol
        @orgRow,orgCol say eText
        @nRow,nCol say ""

    *
    * Return
    *
    case sel = chr(13)
        return

    *
    * Home
    *
    case sel = chr(1)
        nCol = orgCol
        @nRow,nCol say ""

    *
    * End
    *
    case sel = chr(6)
        nCol = orgCol + len(eText)
        if nCol > endCol
            nCol = endCol
        endif
        @nRow,nCol say ""

    *
    * Character
    *
    case sel > chr(31) .and. sel < chr(127)
    @nRow,nCol say sel
    if .not. len(eText) > maxLen
        eText = stuff(eText,(nCol - orgCol)+1,0,sel)
        if nCol < endCol
            nCol = nCol + 1
        endif
    endif
    @orgRow,orgCol say eText
    @nRow,nCol say ""

    *
    * Escape
    *
    case sel = chr(27) .or. key = 0
        eText = ""
        return

    endcase
enddo
return

